/*
 * Copyright 2007-2008 MIYAMOTO Daisuke, jiemamy.org and the Others.
 * Created on 2008/10/22
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.serializer.sax;

import static org.junit.Assert.assertEquals;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.text.StrBuilder;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.jiemamy.JiemamyView;
import org.jiemamy.ReflectionDialectAccessStrategy;
import org.jiemamy.model.Artemis;
import org.jiemamy.model.Jiemamy;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.datatype.IntegerDataTypeDescriptor;
import org.jiemamy.model.datatype.TimestampDataTypeDescriptor;
import org.jiemamy.model.datatype.VarcharDataTypeDescriptor;
import org.jiemamy.model.node.TableModel;

/**
 * {@link ModelInputStream}のテストクラス。
 * @author junichi
 */
public class ModelInputStreamTest {
	
	private static Logger logger = LoggerFactory.getLogger(ModelInputStreamTest.class);
	

	/**
	 * setup
	 * @throws Exception 
	 */
	@Before
	public void setUp() throws Exception {
		Jiemamy.init(Artemis.class, new ReflectionDialectAccessStrategy(), new JiemamyView());
	}
	
	/**
	 * teardown
	 * @throws Exception 
	 */
	@After
	public void tearDown() throws Exception {
		Jiemamy.dispose();
	}
	
	/**
	 * モデルをInputStreamに変えるテスト
	 * @throws Exception
	 */
	@Test
	public void testRead() throws Exception {
		RootModel rootModel = Jiemamy.newRootModel();
		rootModel.setDialectId("org.jiemamy.dialect.mysql.MySqlDialect");
		rootModel.setBeginScript("BEGIN;");
		rootModel.setEndScript("COMMIT;");
		rootModel.setDescription("シリアライゼーションイメージ");
		rootModel.setSchemaName("FOO");
		
		TableModel t = rootModel.newModel(TableModel.class);
		t.setName("T_DEPT");
		t.setDescription("HOGEHOGE");
		
		ColumnModel col1 = rootModel.newModel(ColumnModel.class);
		ColumnModel col2 = rootModel.newModel(ColumnModel.class);
		ColumnModel col3 = rootModel.newModel(ColumnModel.class);
		
		col1.setName("col1");
		col1.setDataTypeDescriptor(new IntegerDataTypeDescriptor());
		col2.setName("col2");
		col2.setDataTypeDescriptor(new VarcharDataTypeDescriptor(16));
		col3.setName("col3");
		col3.setDataTypeDescriptor(new TimestampDataTypeDescriptor());
		
		t.appendModel(col1);
		t.appendModel(col2);
		t.appendModel(col3);
		rootModel.appendModel(t);
		
		// もっと複雑なモデルを出したい時、コメント外す。
//		rootModel = new ApplicationModelCreator().createModel();
		
		StrBuilder sbActual = new StrBuilder();
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(new ModelInputStream(rootModel)));
			String buf;
			while ((buf = reader.readLine()) != null) {
				sbActual.appendln(buf);
			}
		} finally {
			IOUtils.closeQuietly(reader);
		}
		
		StrBuilder sbExpected = new StrBuilder();
		sbExpected.appendln("<?xml version=\"1.0\"?>");
		sbExpected.appendln("<rootModel id=\"" + rootModel.getId().toString() + "\" "
				+ "xsi:schemaLocation=\"http://jiemamy.org/xml/ns/core jiemamy-core.xsd\""
				+ " xmlns=\"http://jiemamy.org/xml/ns/core\""
				+ " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
				+ " xmlns:view=\"http://jiemamy.org/xml/ns/view\">");
		sbExpected.appendln("  <dialect>org.jiemamy.dialect.mysql.MySqlDialect</dialect>");
		sbExpected.appendln("  <schemaName>FOO</schemaName>");
		sbExpected.appendln("  <beginScript>BEGIN;</beginScript>");
		sbExpected.appendln("  <endScript>COMMIT;</endScript>");
		sbExpected.appendln("  <description>シリアライゼーションイメージ</description>");
		sbExpected.appendln("  <domains/>");
		sbExpected.appendln("  <nodes>");
		sbExpected.appendln("    <table id=\"" + t.getId().toString() + "\">");
		sbExpected.appendln("      <name>T_DEPT</name>");
		sbExpected.appendln("      <logicalName/>");
		sbExpected.appendln("      <sourceConnections/>");
		sbExpected.appendln("      <targetConnections/>");
		sbExpected.appendln("      <beginScript/>");
		sbExpected.appendln("      <endScript/>");
		sbExpected.appendln("      <description>HOGEHOGE</description>");
		sbExpected.appendln("      <columns>");
		sbExpected.appendln("        <column id=\"" + t.getColumns().get(0).getId().toString() + "\">");
		sbExpected.appendln("          <name>col1</name>");
		sbExpected.appendln("          <logicalName/>");
		sbExpected.appendln("          <dataType class=\"org.jiemamy.model.datatype.IntegerDataTypeDescriptor\"/>");
		sbExpected.appendln("          <defaultValue/>");
		sbExpected.appendln("          <constraints/>");
		sbExpected.appendln("          <simpleIndex>false</simpleIndex>");
		sbExpected.appendln("          <freeString/>");
		sbExpected.appendln("          <description/>");
		sbExpected.appendln("          <options/>");
		sbExpected.appendln("          <representation>false</representation>");
		sbExpected.appendln("        </column>");
		sbExpected.appendln("        <column id=\"" + t.getColumns().get(1).getId().toString() + "\">");
		sbExpected.appendln("          <name>col2</name>");
		sbExpected.appendln("          <logicalName/>");
		sbExpected.appendln("          <dataType class=\"org.jiemamy.model.datatype.VarcharDataTypeDescriptor\">");
		sbExpected.appendln("            <adapter class=\"org.jiemamy.model.datatype.adapter.SizedDataTypeAdapter\">");
		sbExpected.appendln("              <property name=\"size\">16</property>");
		sbExpected.appendln("            </adapter>");
		sbExpected.appendln("          </dataType>");
		sbExpected.appendln("          <defaultValue/>");
		sbExpected.appendln("          <constraints/>");
		sbExpected.appendln("          <simpleIndex>false</simpleIndex>");
		sbExpected.appendln("          <freeString/>");
		sbExpected.appendln("          <description/>");
		sbExpected.appendln("          <options/>");
		sbExpected.appendln("          <representation>false</representation>");
		sbExpected.appendln("        </column>");
		sbExpected.appendln("        <column id=\"" + t.getColumns().get(2).getId().toString() + "\">");
		sbExpected.appendln("          <name>col3</name>");
		sbExpected.appendln("          <logicalName/>");
		sbExpected.appendln("          <dataType class=\"org.jiemamy.model.datatype.TimestampDataTypeDescriptor\">");
		sbExpected
			.appendln("            <adapter class=\"org.jiemamy.model.datatype.adapter.TimezonedDataTypeAdapter\">");
		sbExpected.appendln("              <property name=\"withTimezone\">false</property>");
		sbExpected.appendln("            </adapter>");
		sbExpected.appendln("          </dataType>");
		sbExpected.appendln("          <defaultValue/>");
		sbExpected.appendln("          <constraints/>");
		sbExpected.appendln("          <simpleIndex>false</simpleIndex>");
		sbExpected.appendln("          <freeString/>");
		sbExpected.appendln("          <description/>");
		sbExpected.appendln("          <options/>");
		sbExpected.appendln("          <representation>false</representation>");
		sbExpected.appendln("        </column>");
		sbExpected.appendln("      </columns>");
		sbExpected.appendln("    </table>");
		sbExpected.appendln("  </nodes>");
		sbExpected.appendln("  <connections/>");
		sbExpected.appendln("  <insertDataSetModels/>");
		sbExpected.appendln("  <view:diagramPresentations/>");
		sbExpected.appendln("</rootModel>");
		
		logger.info("================ actual ================================================================");
		logger.info(sbActual.toString());
		logger.info("================ expected ==============================================================");
		logger.info(sbExpected.toString());
		assertEquals(sbExpected.toString(), sbActual.toString());
	}
}
