package org.jiemamy.serializer;

import java.util.ArrayList;
import java.util.List;

import org.jiemamy.JiemamyView;
import org.jiemamy.exception.ElementNotFoundException;
import org.jiemamy.exception.TooManyElementsException;
import org.jiemamy.exception.UnexpectedConditionError;
import org.jiemamy.model.Artemis;
import org.jiemamy.model.DiagramPresentationModel;
import org.jiemamy.model.DiagramPresentations;
import org.jiemamy.model.Jiemamy;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.connection.ForeignKeyModel;
import org.jiemamy.model.connection.ForeignKeyModel.InitiallyCheckTime;
import org.jiemamy.model.connection.ForeignKeyModel.ReferentialAction;
import org.jiemamy.model.constraint.NotNullConstraintModel;
import org.jiemamy.model.constraint.PrimaryKeyConstraintModel;
import org.jiemamy.model.constraint.UniqueConstraintModel;
import org.jiemamy.model.dataset.InsertDataSetModel;
import org.jiemamy.model.dataset.RecordModel;
import org.jiemamy.model.datatype.DomainModel;
import org.jiemamy.model.datatype.IntegerDataTypeDescriptor;
import org.jiemamy.model.datatype.NumericDataTypeDescriptor;
import org.jiemamy.model.datatype.TimestampDataTypeDescriptor;
import org.jiemamy.model.datatype.VarcharDataTypeDescriptor;
import org.jiemamy.model.datatype.adapter.SerialDataTypeAdapter;
import org.jiemamy.model.geometory.JmPoint;
import org.jiemamy.model.geometory.JmRectangle;
import org.jiemamy.model.node.StickyModel;
import org.jiemamy.model.node.TableModel;
import org.jiemamy.model.node.ViewModel;

/**
 * アプリケーション用のモデルを生成するクラス。
 * 
 * @author j5ik2o
 */
public class ApplicationModelCreator {
	
	private static final String DIALECT_CLASS_NAME = "org.jiemamy.dialect.mysql.MySqlDialect";
	

	/**
	 * モデル生成処理が正常に終わるかどうかチェックするためのmainメソッド。
	 * @param args
	 */
	public static void main(String[] args) {
		Jiemamy.init(Artemis.class, new JiemamyView());
		RootModel rootModel = new ApplicationModelCreator().createModel();
		System.out.println(rootModel);
		Jiemamy.dispose();
	}
	

	private RootModel rootModel;
	

	/**
	 * テスト用モデルを生成する。
	 * @return モデル
	 */
	public RootModel createModel() {
		int offset = 50;
		
		rootModel = Jiemamy.newRootModel();
		rootModel.setDialectId(DIALECT_CLASS_NAME);
		rootModel.setBeginScript("BEGIN;");
		rootModel.setEndScript("COMMIT;");
		rootModel.setDescription("シリアライゼーションイメージ");
		rootModel.setSchemaName("FOO");
		
		// ドメインの生成
		DomainModel idDomain = rootModel.newModel(DomainModel.class);
		idDomain.setName("ID");
		idDomain.setDataTypeDescriptor(new IntegerDataTypeDescriptor());
		idDomain.addConstraint(rootModel.newModel(NotNullConstraintModel.class));
		idDomain.addConstraint(rootModel.newModel(UniqueConstraintModel.class));
		idDomain.registerAdapter(new SerialDataTypeAdapter());
		rootModel.appendModel(idDomain);
		
		DomainModel nameDomain = rootModel.newModel(DomainModel.class);
		nameDomain.setName("NAME");
		nameDomain.setDataTypeDescriptor(new VarcharDataTypeDescriptor(32));
		nameDomain.setDescription("人名用の型です。");
		rootModel.appendModel(nameDomain);
		
		// ノードの生成
		TableModel deptTable = createDeptTable(idDomain, nameDomain);
		rootModel.appendModel(deptTable);
		TableModel empTable = createEmpTable(idDomain, nameDomain);
		rootModel.appendModel(empTable);
		
		ViewModel highSalView = rootModel.newModel(ViewModel.class);
		highSalView.setName("V_HIGH_SAL_EMP");
		highSalView.setDefinition("SELECT * FROM T_EMP WHERE SAL > 2000;");
		highSalView.setLogicalName("高給取り");
		rootModel.appendModel(highSalView);
		
		StickyModel sticky = rootModel.newModel(StickyModel.class);
		sticky.setContents("メモーー");
		rootModel.appendModel(sticky);
		
		// コネクションの生成・追加
		ForeignKeyModel fkEmpEmp = createForeignKey(empTable, empTable, "MGR_ID");
		fkEmpEmp.setOnDelete(ReferentialAction.SET_NULL);
		fkEmpEmp.setDeferrable(true);
		fkEmpEmp.setInitiallyCheckTime(InitiallyCheckTime.DEFERRED);
		rootModel.appendModel(fkEmpEmp);
		
		ForeignKeyModel fkEmpDept = createForeignKey(deptTable, empTable, "DEPT_ID");
		rootModel.appendModel(fkEmpDept);
		
		// ダイアグラム表現の生成・追加(1)
		DiagramPresentationModel presentation = rootModel.newModel(DiagramPresentationModel.class);
		presentation.setName("全部表示する");
		presentation.getNodeLayouts().put(empTable, new JmRectangle(360, 60));
		presentation.getNodeLayouts().put(deptTable, new JmRectangle(60, 60));
		presentation.getNodeLayouts().put(highSalView, new JmRectangle(60, 270));
		presentation.getNodeLayouts().put(sticky, new JmRectangle(360, 270));
		
		List<JmPoint> bendpoints = new ArrayList<JmPoint>();
		JmRectangle rect = presentation.getNodeLayouts().get(empTable);
		bendpoints.add(0, new JmPoint(Math.max(rect.x - offset, 0), rect.y));
		bendpoints.add(1, new JmPoint(rect.x, Math.max(rect.y - offset, 0)));
		presentation.getConnectionLayouts().put(fkEmpEmp, bendpoints);
		presentation.getConnectionLayouts().put(fkEmpDept, new ArrayList<JmPoint>()); // bendpount無し
		rootModel.getAdapter(DiagramPresentations.class).appendModel(presentation);
		
		// ダイアグラム表現の生成・追加(2)
		presentation = rootModel.newModel(DiagramPresentationModel.class);
		presentation.setName("一部表示する");
		presentation.getNodeLayouts().put(empTable, new JmRectangle(60, 60));
		presentation.getNodeLayouts().put(highSalView, new JmRectangle(270, 270));
		presentation.getConnectionLayouts().put(fkEmpDept, new ArrayList<JmPoint>()); // bendpount無し
		rootModel.getAdapter(DiagramPresentations.class).appendModel(presentation);
		
		// データセットの生成・追加(1)
		InsertDataSetModel dataSetEn = rootModel.newModel(InsertDataSetModel.class);
		dataSetEn.setName("データ群en");
		dataSetEn.getRecords().put(deptTable, createDeptDataSetEn(deptTable));
		dataSetEn.getRecords().put(empTable, createEmpDataSetEn(empTable));
		rootModel.appendModel(dataSetEn);
		
		// データセットの生成・追加(2)
		InsertDataSetModel dataSetJa = rootModel.newModel(InsertDataSetModel.class);
		dataSetJa.setName("データ群ja");
		dataSetJa.getRecords().put(deptTable, createDeptDataSetJa(deptTable));
		dataSetJa.getRecords().put(empTable, createEmpDataSetJa(empTable));
		rootModel.appendModel(dataSetJa);
		
		return rootModel;
	}
	
	private List<RecordModel> createDeptDataSetEn(TableModel deptTable) {
		List<RecordModel> result = new ArrayList<RecordModel>();
		
		try {
			RecordModel record = rootModel.newModel(RecordModel.class);
			
			record.getValues().put(deptTable.getColumn("ID"), "1");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "10");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "ACCOUNTING");
			record.getValues().put(deptTable.getColumn("LOC"), "NEW YORK");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(deptTable.getColumn("ID"), "2");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "20");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "RESEARCH");
			record.getValues().put(deptTable.getColumn("LOC"), "DALLAS");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(deptTable.getColumn("ID"), "3");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "30");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "SALES");
			record.getValues().put(deptTable.getColumn("LOC"), "CHICAGO");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(deptTable.getColumn("ID"), "4");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "40");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "OPERATIONS");
			record.getValues().put(deptTable.getColumn("LOC"), "BOSTON");
			result.add(record);
		} catch (TooManyElementsException e) {
			throw new UnexpectedConditionError("test code error", e);
		}
		
		return result;
	}
	
	private List<RecordModel> createDeptDataSetJa(TableModel deptTable) {
		List<RecordModel> result = new ArrayList<RecordModel>();
		
		try {
			RecordModel record = rootModel.newModel(RecordModel.class);
			
			record.getValues().put(deptTable.getColumn("ID"), "1");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "10");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "経理部");
			record.getValues().put(deptTable.getColumn("LOC"), "広島");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(deptTable.getColumn("ID"), "2");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "20");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "研究開発部");
			record.getValues().put(deptTable.getColumn("LOC"), "京都");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(deptTable.getColumn("ID"), "3");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "30");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "営業部");
			record.getValues().put(deptTable.getColumn("LOC"), "東京");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(deptTable.getColumn("ID"), "4");
			record.getValues().put(deptTable.getColumn("DEPT_NO"), "40");
			record.getValues().put(deptTable.getColumn("DEPT_NAME"), "経営本部");
			// locは指定せず、特定しない状態
			result.add(record);
		} catch (TooManyElementsException e) {
			throw new UnexpectedConditionError("test code error", e);
		}
		
		return result;
	}
	
	/**
	 * DEPTテーブルの生成。
	 * @param idDomain IDの型として使用するドメイン
	 * @param nameDomain 名称として使用するドメイン
	 * @return DEPTテーブル
	 */
	private TableModel createDeptTable(DomainModel idDomain, DomainModel nameDomain) {
		TableModel deptTable = rootModel.newModel(TableModel.class);
		deptTable.setName("T_DEPT");
		deptTable.setBeginScript("/* test begin script */");
		deptTable.setDescription("部署マスタです。");
		
		ColumnModel idColumn = rootModel.newModel(ColumnModel.class);
		idColumn.setName("ID");
		idColumn.setDataTypeDescriptor(idDomain);
		idColumn.addConstraint(rootModel.newModel(PrimaryKeyConstraintModel.class));
		idColumn.setLogicalName("部署ID");
		deptTable.appendModel(idColumn);
		
		ColumnModel noColumn = rootModel.newModel(ColumnModel.class);
		noColumn.setName("DEPT_NO");
		noColumn.setDataTypeDescriptor(new IntegerDataTypeDescriptor());
		noColumn.setLogicalName("部署番号");
		deptTable.appendModel(noColumn);
		
		ColumnModel deptNameColumn = rootModel.newModel(ColumnModel.class);
		deptNameColumn.setName("DEPT_NAME");
		deptNameColumn.setDataTypeDescriptor(new VarcharDataTypeDescriptor(20));
		deptNameColumn.setLogicalName("部署名");
		deptNameColumn.setRepresentation(true);
		deptTable.appendModel(deptNameColumn);
		
		ColumnModel locColumn = rootModel.newModel(ColumnModel.class);
		locColumn.setName("LOC");
		locColumn.setDataTypeDescriptor(new VarcharDataTypeDescriptor(20));
		locColumn.setLogicalName("ロケーション");
		locColumn.setDefaultValue("secret");
		deptTable.appendModel(locColumn);
		
		return deptTable;
	}
	
	private List<RecordModel> createEmpDataSetEn(TableModel empTable) {
		List<RecordModel> result = new ArrayList<RecordModel>();
		
		try {
			RecordModel record = rootModel.newModel(RecordModel.class);
			
			record.getValues().put(empTable.getColumn("ID"), "1");
			record.getValues().put(empTable.getColumn("EMP_NO"), "10");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "SMITH");
			record.getValues().put(empTable.getColumn("MGR_ID"), "3");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2003-02-01");
			record.getValues().put(empTable.getColumn("SAL"), "40");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "3");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "2");
			record.getValues().put(empTable.getColumn("EMP_NO"), "20");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "ALLEN");
			record.getValues().put(empTable.getColumn("MGR_ID"), "3");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2000-03-04");
			record.getValues().put(empTable.getColumn("SAL"), "50");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "4");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "3");
			record.getValues().put(empTable.getColumn("EMP_NO"), "30");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "WARD");
			record.getValues().put(empTable.getColumn("MGR_ID"), null);
			record.getValues().put(empTable.getColumn("HIREDATE"), "1993-12-05");
			record.getValues().put(empTable.getColumn("SAL"), "60");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "4");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "4");
			record.getValues().put(empTable.getColumn("EMP_NO"), "40");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "JONES");
			record.getValues().put(empTable.getColumn("MGR_ID"), "2");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2007-04-01");
			record.getValues().put(empTable.getColumn("SAL"), "36");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "2");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "5");
			record.getValues().put(empTable.getColumn("EMP_NO"), "50");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "MARTIN");
			record.getValues().put(empTable.getColumn("MGR_ID"), "1");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2002-05-30");
			record.getValues().put(empTable.getColumn("SAL"), "30");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "3");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "6");
			record.getValues().put(empTable.getColumn("EMP_NO"), "60");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "BLAKE");
			record.getValues().put(empTable.getColumn("MGR_ID"), "3");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2007-04-01");
			record.getValues().put(empTable.getColumn("SAL"), "25");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "2");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "7");
			record.getValues().put(empTable.getColumn("EMP_NO"), "70");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "CLARK");
			record.getValues().put(empTable.getColumn("MGR_ID"), "1");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2004-09-01");
			record.getValues().put(empTable.getColumn("SAL"), "30");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "1");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "8");
			record.getValues().put(empTable.getColumn("EMP_NO"), "80");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "SCOTT");
			record.getValues().put(empTable.getColumn("MGR_ID"), "4");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2008-03-01");
			record.getValues().put(empTable.getColumn("SAL"), "25");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "2");
			result.add(record);
		} catch (TooManyElementsException e) {
			throw new UnexpectedConditionError("test code error", e);
		}
		
		return result;
	}
	
	private List<RecordModel> createEmpDataSetJa(TableModel empTable) {
		List<RecordModel> result = new ArrayList<RecordModel>();
		
		try {
			RecordModel record = rootModel.newModel(RecordModel.class);
			
			record.getValues().put(empTable.getColumn("ID"), "1");
			record.getValues().put(empTable.getColumn("EMP_NO"), "10");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "山本");
			record.getValues().put(empTable.getColumn("MGR_ID"), "3");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2003-02-01");
			record.getValues().put(empTable.getColumn("SAL"), "40");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "3");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "2");
			record.getValues().put(empTable.getColumn("EMP_NO"), "20");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "内海");
			record.getValues().put(empTable.getColumn("MGR_ID"), "3");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2000-03-04");
			record.getValues().put(empTable.getColumn("SAL"), "50");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "4");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "3");
			record.getValues().put(empTable.getColumn("EMP_NO"), "30");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "村瀬");
			record.getValues().put(empTable.getColumn("MGR_ID"), null);
			record.getValues().put(empTable.getColumn("HIREDATE"), "1993-12-05");
			record.getValues().put(empTable.getColumn("SAL"), "60");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "4");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "4");
			record.getValues().put(empTable.getColumn("EMP_NO"), "40");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "近藤");
			record.getValues().put(empTable.getColumn("MGR_ID"), "2");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2007-04-01");
			record.getValues().put(empTable.getColumn("SAL"), "36");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "2");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "5");
			record.getValues().put(empTable.getColumn("EMP_NO"), "50");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "小峰");
			record.getValues().put(empTable.getColumn("MGR_ID"), "1");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2002-05-30");
			record.getValues().put(empTable.getColumn("SAL"), "30");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "3");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "6");
			record.getValues().put(empTable.getColumn("EMP_NO"), "60");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "三浦");
			record.getValues().put(empTable.getColumn("MGR_ID"), "3");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2007-04-01");
			record.getValues().put(empTable.getColumn("SAL"), "25");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "2");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "7");
			record.getValues().put(empTable.getColumn("EMP_NO"), "70");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "前島");
			record.getValues().put(empTable.getColumn("MGR_ID"), "1");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2004-09-01");
			record.getValues().put(empTable.getColumn("SAL"), "30");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "1");
			result.add(record);
			
			record = rootModel.newModel(RecordModel.class);
			record.getValues().put(empTable.getColumn("ID"), "8");
			record.getValues().put(empTable.getColumn("EMP_NO"), "80");
			record.getValues().put(empTable.getColumn("EMP_NAME"), "島崎");
			record.getValues().put(empTable.getColumn("MGR_ID"), "4");
			record.getValues().put(empTable.getColumn("HIREDATE"), "2008-03-01");
			record.getValues().put(empTable.getColumn("SAL"), "25");
			record.getValues().put(empTable.getColumn("DEPT_ID"), "2");
			result.add(record);
		} catch (TooManyElementsException e) {
			throw new UnexpectedConditionError("test code error", e);
		}
		
		return result;
	}
	
	/**
	 * EMPテーブルの生成
	 * @param idDomain IDの型として使用するドメイン
	 * @param nameDomain 名称として使用するドメイン
	 * @return EMPテーブル
	 */
	private TableModel createEmpTable(DomainModel idDomain, DomainModel nameDomain) {
		TableModel empTable = rootModel.newModel(TableModel.class);
		empTable.setName("T_EMP");
		empTable.setLogicalName("従業員");
		empTable.setBeginScript("/* test end script */");
		empTable.setDescription("従業員マスタです。");
		
		ColumnModel idColumn = rootModel.newModel(ColumnModel.class);
		idColumn.setName("ID");
		idColumn.setDataTypeDescriptor(idDomain);
		idColumn.addConstraint(rootModel.newModel(PrimaryKeyConstraintModel.class));
		idColumn.setLogicalName("従業員ID");
		empTable.appendModel(idColumn);
		
		ColumnModel noColumn = rootModel.newModel(ColumnModel.class);
		noColumn.setName("EMP_NO");
		noColumn.setDataTypeDescriptor(new IntegerDataTypeDescriptor());
		noColumn.addConstraint(rootModel.newModel(NotNullConstraintModel.class));
		noColumn.setLogicalName("従業員番号");
		empTable.appendModel(noColumn);
		
		ColumnModel nameColumn = rootModel.newModel(ColumnModel.class);
		nameColumn.setName("EMP_NAME");
		nameColumn.setDataTypeDescriptor(nameDomain);
		nameColumn.addConstraint(rootModel.newModel(NotNullConstraintModel.class));
		nameColumn.setLogicalName("従業員名");
		nameColumn.setDefaultValue("no name");
		nameColumn.setSimpleIndex(true);
		nameColumn.setRepresentation(true);
		empTable.appendModel(nameColumn);
		
		ColumnModel mgrColumn = rootModel.newModel(ColumnModel.class);
		mgrColumn.setName("MGR_ID");
		mgrColumn.setDataTypeDescriptor(new IntegerDataTypeDescriptor());
		nameColumn.setLogicalName("上司ID");
		empTable.appendModel(mgrColumn);
		
		ColumnModel hireColumn = rootModel.newModel(ColumnModel.class);
		hireColumn.setName("HIREDATE");
		hireColumn.setDataTypeDescriptor(new TimestampDataTypeDescriptor());
		hireColumn.addConstraint(rootModel.newModel(NotNullConstraintModel.class));
		empTable.appendModel(hireColumn);
		
		ColumnModel salColumn = rootModel.newModel(ColumnModel.class);
		salColumn.setName("SAL");
		salColumn.setDataTypeDescriptor(new NumericDataTypeDescriptor(7, 2));
		salColumn.addConstraint(rootModel.newModel(NotNullConstraintModel.class));
		empTable.appendModel(salColumn);
		
		ColumnModel deptColumn = rootModel.newModel(ColumnModel.class);
		deptColumn.setName("DEPT_ID");
		deptColumn.setDataTypeDescriptor(new IntegerDataTypeDescriptor());
		deptColumn.addConstraint(rootModel.newModel(NotNullConstraintModel.class));
		empTable.appendModel(deptColumn);
		
		return empTable;
	}
	
	/**
	 * 外部キーを生成する。
	 * このメソッドは、複合キーテーブルを参照する外部キーは生成できない。
	 * @param target 参照先テーブル
	 * @param source 制約を受けるテーブル
	 * @param sourceColumnName 制約を受けるカラムのカラム名 
	 * @return 外部キー
	 */
	private ForeignKeyModel createForeignKey(TableModel target, TableModel source, String sourceColumnName) {
		String[] sourceColumnNames = {
			sourceColumnName
		};
		
		ForeignKeyModel fk = rootModel.newModel(ForeignKeyModel.class);
		fk.setSource(source);
		fk.setTarget(target);
		fk.autoDenominate(rootModel);
		fk.createMappings(rootModel, false);
		for (int i = 0; i < sourceColumnNames.length; i++) {
			try {
				ColumnModel constraintColumn = source.getColumn(sourceColumnNames[i]);
				fk.getMappings().get(i).setConstraintColumn(constraintColumn);
				break;
			} catch (TooManyElementsException e) {
				throw new UnexpectedConditionError("test code error", e);
			} catch (ElementNotFoundException e) {
				// ignore
			}
		}
		
		return fk;
	}
}
