/*
 * Copyright 2007-2008 MIYAMOTO Daisuke, jiemamy.org and the Others.
 * Created on 2008/06/09
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.serializer;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.CharEncoding;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.junit.After;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.jiemamy.JiemamyView;
import org.jiemamy.model.Artemis;
import org.jiemamy.model.Jiemamy;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.connection.AbstractConnectionModel;
import org.jiemamy.model.datatype.DomainModel;
import org.jiemamy.model.node.StickyModel;
import org.jiemamy.model.node.TableModel;
import org.jiemamy.model.node.ViewModel;

/**
 * モデルのシリアライズ全体のテスト。
 * コードで作ったモデル -(serialize)-> XML -(deserialize-> 復元モデルを比較して同一性をチェックする。
 * @author daisuke
 */
public class SerializationTest {
	
	private static Logger logger = LoggerFactory.getLogger(SerializationTest.class);
	
	/** JiemamySerializer */
	private JiemamySerializer jiemamySerializer;
	

	/**
	 * setup
	 * @throws Exception 
	 */
	@Before
	public void setUp() throws Exception {
		Jiemamy.init(Artemis.class, new JiemamyView());
//		jiemamySerializer = new JiemamySaxSerializer();
		jiemamySerializer = new JiemamyStaxSerializer();
	}
	
	/**
	 * teardown
	 * @throws Exception 
	 */
	@After
	public void tearDown() throws Exception {
		jiemamySerializer = null;
		Jiemamy.dispose();
	}
	
	/**
	 * test
	 * @throws Exception
	 */
	@Test
	public void test01_とりあえず書き込んでみる() throws Exception {
		ByteArrayOutputStream out = null;
		RootModel rootModel = new ApplicationModelCreator().createModel();
		
		try {
			out = new ByteArrayOutputStream();
			jiemamySerializer.serialize(rootModel, out);
			logger.info(out.toString(CharEncoding.UTF_8));
		} finally {
			IOUtils.closeQuietly(out);
		}
	}
	
	/**
	 * test
	 * @throws Exception
	 */
	@Test
//	@Ignore
	public void test02_とりあえず読み込んでみる() throws Exception {
		InputStream in = null;
		RootModel rootModel;
		try {
			in = SerializationTest.class.getResourceAsStream("/sample.xml");
			rootModel = jiemamySerializer.deserialize(in);
		} finally {
			IOUtils.closeQuietly(in);
		}
		
		assertThat(rootModel, is(notNullValue()));
		
		logger.info("RootModel(Id):" + rootModel.getId());
		logger.info("RootModel(Dialect):" + rootModel.getDialectId());
		logger.info("RootModel(BeginScript):" + rootModel.getBeginScript());
		logger.info("RootModel(EndScript):" + rootModel.getEndScript());
		logger.info("RootModel(Description):" + rootModel.getDescription());
		logger.info("RootModel(SchemaName):" + rootModel.getSchemaName());
		
		logger.info("***Domain");
		List<DomainModel> domains = rootModel.getDomains();
		for (DomainModel domain : domains) {
			logger.info("  " + ToStringBuilder.reflectionToString(domain));
		}
		
		logger.info("***Table");
		List<TableModel> tables = rootModel.getNodes(TableModel.class);
		for (TableModel table : tables) {
			logger.info("  Table:" + table.getId() + "," + table.getName() + "," + table.getLogicalName() + ","
					+ table.getDescription());
			
			for (ColumnModel column : table.getColumns()) {
				logger.info("    " + column);
			}
		}
		
		logger.info("***View");
		List<ViewModel> views = rootModel.getNodes(ViewModel.class);
		for (ViewModel view : views) {
			logger.info("  View:" + view);
		}
		
		logger.info("***Sticky");
		List<StickyModel> stickies = rootModel.getNodes(StickyModel.class);
		for (StickyModel sticky : stickies) {
			logger.info("  Sticky:" + sticky);
		}
		
		logger.info("***Connection");
		List<AbstractConnectionModel> connections = rootModel.getConnections();
		for (AbstractConnectionModel conn : connections) {
			logger.info("  Connection:" + conn);
		}
	}
	
	/**
	 * シリアライザの動作チェック。
	 * @throws Exception 
	 */
	@Test
	@Ignore
	public void test03_シリアライザの動作チェック() throws Exception {
		RootModel rootModel = new ApplicationModelCreator().createModel();
		
		PipedInputStream serializeIn1 = null;
		PipedInputStream serializeIn2 = null;
		InputStream referenceIn1 = null;
		InputStream referenceIn2 = null;
		InputStream referenceIn3 = null;
		PipedOutputStream out1 = null;
		PipedOutputStream out2 = null;
		
		try {
			// とりあえず表示
			serializeIn1 = new PipedInputStream();
			out1 = new PipedOutputStream(serializeIn1);
			jiemamySerializer.serialize(rootModel, out1);
			
			logger.info("serialized=");// + IOUtils.toString(serializeIs1));
			print(serializeIn1);
			referenceIn1 = SerializationTest.class.getResourceAsStream("/sample.xml");
			logger.info("reference=");// + IOUtils.toString(referenceIs1));
			print(referenceIn1);
			
			// シリアライズ結果と参照ファイルの同一性試験
			serializeIn2 = new PipedInputStream();
			out2 = new PipedOutputStream(serializeIn2);
			referenceIn2 = SerializationTest.class.getResourceAsStream("/sample.xml");
			jiemamySerializer.serialize(rootModel, out2);
//			assertTrue(IOUtils.contentEquals(serializeIs2, referenceIs2));
			
			// this.rootModel と deserialized の同一性試験
			referenceIn3 = SerializationTest.class.getResourceAsStream("/sample.xml");
			RootModel deserialized = jiemamySerializer.deserialize(referenceIn3);
			
			logger.info("======== ORIGINAL");
			logger.info(ToStringBuilder.reflectionToString(rootModel, ToStringStyle.SHORT_PREFIX_STYLE));
			logger.info("======== DESERIALIZED");
			logger.info(ToStringBuilder.reflectionToString(deserialized, ToStringStyle.SHORT_PREFIX_STYLE));
			
			// TODO [CORE-22] モデルに関するequalsを実装する。
			assertTrue(ToStringBuilder.reflectionToString(rootModel, ToStringStyle.SHORT_PREFIX_STYLE).equals(
					ToStringBuilder.reflectionToString(deserialized, ToStringStyle.SHORT_PREFIX_STYLE)));
		} finally {
			IOUtils.closeQuietly(serializeIn1);
			IOUtils.closeQuietly(serializeIn2);
			IOUtils.closeQuietly(referenceIn1);
			IOUtils.closeQuietly(referenceIn2);
			IOUtils.closeQuietly(referenceIn3);
			IOUtils.closeQuietly(out1);
			IOUtils.closeQuietly(out2);
		}
	}
	
	/**
	 * {@link InputStream}の内容をlog出力する。
	 * @param in
	 * @throws IOException 
	 */
	private void print(InputStream in) throws IOException {
		final BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		String buf;
		while ((buf = reader.readLine()) != null) {
			logger.info(buf);
		}
	}
	
}
